/*****************************************************************************/
/* Betriebssysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*                          B E L L R I N G E R                              */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* Der "Gloeckner" wird im Rahmen der Timer-Epilog Bearbeitung aktiviert.    */
/* Er prueft, ob irgendwelche Glocken (Bell-Objekte) laeuten muessen. Die    */
/* Glocken befinden sich in einer Queue, die der Gloeckner verwaltet.        */
/*****************************************************************************/

/* INCLUDES */

#include "meeting/bellringer.h"
#include "meeting/bell.h"

/* METHODEN */

// CHECK: Prueft, ob Glocken zu laeuten sind und tut es gegebenenfalls
void Bellringer::check ()
 {
   Bell *bell;

   bell = (Bell *) first ();
   if (bell)
    {
      bell->tick ();
      if (bell->run_down ())
         ring_the_bells ();
    }
 }

// JOB: Eine Glocke wird dem Gloeckner ueberantwortet. Sie muss nach
//      einer bestimmten Zeit gelaeutet werden.
void Bellringer::job (Bell *bell, int ticks)
 {
   Bell* curr;
   Bell* pred;

   if (!first ())  // Die Warteschlange ist leer
    {
      enqueue (bell);
    }
   else
    {
      // Um die Arbeit des Gloeckners zu erleichtern, ist die Warteschlange
      // der Glocken nach Wartezeit sortiert. Ausserdem wird keine absolute
      // Wartezeit, sondern stets nur die Differenz zum Vorgaenger einge-
      // tragen. Dadurch muss beim "Ticken" der Uhren immer nur der Zaehler
      // der ersten Uhr veraendert werden.

      pred = (Bell *) 0;
      curr = (Bell *) first ();
      while (curr && ticks >= curr->wait ())
       {
         ticks -= curr->wait ();
         pred = curr;
	 curr = (Bell *) pred->next;
       }
      if (pred)
        insert_after (pred, bell);
      else
        insert_first (bell);

      if (curr)
        curr->wait (curr->wait () - ticks);
    }

   bell->wait (ticks);
 }

// CANCEL: Eine bestimmte Glocke soll nun doch nicht gelaeutet werden
void Bellringer::cancel (Bell *bell)
 {
   Bell *curr;

   curr = (Bell *) first ();
   
   while (curr && curr != bell)
      curr = (Bell *) curr->next;

   if (curr == bell)   // Hoffentlich war das Objekt auch auf der Warteliste.
    {
      curr = (Bell *)curr->next;
      if (curr)
	 curr->wait (curr->wait () + bell->wait ());
      remove (bell);
    }
 }   
   
// RING_THE_BELLS: Laeuten der Glocken

void Bellringer::ring_the_bells ()
 {
   Bell *bell;

   while ((bell = (Bell *) first ()) && bell->run_down ())
      {
	 dequeue ();
	 bell->ring ();
      }
 }

