/*****************************************************************************/
/* Betriebssysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*                               B U Z Z E R                                 */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* Mit Hilfe des "Weckers" koennen Prozesse eine bestimmte Zeit lang         */
/* schlafen und sich dann wecken lassen.                                     */
/*****************************************************************************/

/* INCLUDES */

#include "meeting/buzzer.h"
#include "meeting/bellringer.h"
#include "syscall/guarded_organizer.h"
#include "device/watch.h"

/* GLOBALE VARIABLEN */

extern Bellringer bellringer;
extern Guarded_Organizer organizer;
extern Watch watch;

/* METHODEN */

// DESTRUKTOR: Den Gloeckner informieren, das er hier nicht mehr laeuten
//             muss
Buzzer::~Buzzer ()
 {
   if (on)
    {
      bellringer.cancel (this);
      on = false;
    }
 }

// RING: Die Wartezeit ist abgelaufen.
void Buzzer::ring ()
 {
   Customer *customer;

   while ((customer = (Customer *) dequeue ()))
      organizer.Organizer::wakeup (*customer);
   on = false;
 }

// SET: Stellen des Weckers
void Buzzer::set (int ms)
 {
   // Umrechnen von Millisekunden in Ticks
   int ticks = ms * 1000 / watch.interval ();
   bellringer.job (this, ticks);
   on = true;
 }

// SLEEP: Schlafen bis der Wecker klingelt.
void Buzzer::sleep ()
 {
   Customer *life;

   life = (Customer *)organizer.Organizer::active ();
   enqueue (life);
   organizer.Organizer::block (*life, *this);
 }
